//=============================================================================
// ClockInfo by Revelation.
//
// This actor brings a fully functional, analogue clock to UT.
// To make it work, it must be used with PointerMovers representing the clock's
// hour hand, minute hand and second hand.
// Notice that you can operate more than one clock with a single ClockInfo by
// giving the PointerMovers equal Tags.
// What all the variables do is explained in the script below. For an example,
// have a look a my ClockDemo map!
//=============================================================================
class ClockInfo extends Keypoint;

#exec TEXTURE IMPORT NAME=ClockIcon FILE=textures\ClockIcon.pcx GROUP=Icons FLAGS=2

var PointerMover Hour, Minute, Second; // Variables adressing the three different PointerMovers for HourHand, MinuteHand and SecondHand
var int hx, h, m, s, CountUp; // Script-internal counters used for initially adjusting the clock and checking for events

var() bool bUseRealTime; // If true, the clock starts with your current system-time. Needs to be tested online.
var() int InitialTimeHours, InitialTimeMinutes, InitialTimeSeconds; // Define the clock's starting time, if bUseRealTime is set to false. Values must be 0-23 for InitialTimeHours and 0-59 for InitialTimeMinutes and InitialTimeSeconds.
var() name TagHourHand, TagMinuteHand, TagSecondHand; // Tags of the PointerMovers representing hour hand, minute hand and second hand

struct TE
{
	var() int EventTimeHour, EventTimeMinute, EventTimeSecond; // Specifies the exact time when one of the TimedEvents should be triggered.
	var() name EventAtThisTime; // The event to be called at the given time
};

var(ClockInfoEvents) TE TimedEvents[24]; // 24 sets of TimedEvents should be enough, right?^^
var(ClockInfoEvents) name EventEveryFullHour, EventEveryFullMinute; // Events to be called periodically at every full hour and minute

var(ClockInfoSounds) sound FullHourSound, FullMinuteSound, TickingSound; // Sounds to be played at every full hour, minute and second. The sound played every second should only be used for the clock's "ticking", therefor it's called TickingSound.


function PostBeginPlay()
{
	if(bUseRealTime)
	{
		hx = Level.Hour;
		m = Level.Minute;
		s = Level.Second;
	}
	else
	{
		hx = InitialTimeHours;
		m = InitialTimeMinutes;
		s = InitialTimeSeconds;
	}

	if(hx >= 12)
		h = hx - 12;
	else
		h = hx;

	CountUp = 3600*hx + 60*m + s;

	if(TagHourHand != '')
		foreach allactors (class'PointerMover', Hour, TagHourHand)
		{
			Hour.WaitTime = 60;
			Hour.RotHere = h*5461 + m*91;
			Hour.RotNew = 91;
			Hour.StartRot();
		}

	if(TagMinuteHand != '')
		foreach allactors (class'PointerMover', Minute, TagMinuteHand)
		{
			Minute.FS = FullHourSound;
			Minute.FE = EventEveryFullHour;
			Minute.WaitTime = 5;
			Minute.RotHere = m*1092 + (s/5)*91;
			Minute.RotNew = 91;
			Minute.StartRot();
		}

	if(TagSecondHand != '')
		foreach allactors (class'PointerMover', Second, TagSecondHand)
		{
			Second.TS = TickingSound;
			Second.FS = FullMinuteSound;
			Second.FE = EventEveryFullMinute;
			Second.WaitTime = 1;
			Second.RotHere = s*1092;
			Second.RotNew = 1092;
			Second.StartRot();
		}

	SetTimer(1, False);
}

function Timer()
{
	CountUp += 1;
	if(CountUp == 86400)
		CountUp = 0;
	CheckForEvents();
}

function CheckForEvents()
{
	local int i;
	local actor a;

	for(i=0; i<24; i++)
		if(TimedEvents[i].EventAtThisTime != '')
			if(CountUp == ((TimedEvents[i].EventTimeHour)*3600 + (TimedEvents[i].EventTimeMinute)*60 + TimedEvents[i].EventTimeSecond))
				foreach allactors(class 'Actor', a, TimedEvents[i].EventAtThisTime)
					a.Trigger(Self, None);

	SetTimer(1, False);
}

defaultproperties
{
				bStatic=False
				Texture=Texture'LNN.Icons.ClockIcon'
				SoundRadius=12
				SoundVolume=128
				TransientSoundRadius=1.000000
}
